from slugify import slugify
from pathlib import Path
from orji.utils import random_5_digit_number
import re


class OrjiError(Exception):
    pass


class TextChunk:
    def __init__(self, text):
        self.text = text

    @property
    def markdown(self):
        text = self.text
        text = re.sub(
            re.compile(r"\[\[(.*?)\]\[(.*?)\]\]"),
            r"[\2](\1)",
            text,
        )
        text = text.replace("\n+ ", "\n* ")
        return text.strip()

    @property
    def latexed(self):
        text = self.text
        text = re.sub(
            re.compile(r"\[\[(.*?)\]\[(.*?)\]\]"),
            r"\\href{\1}{\2}",
            text,
        )
        text = text.replace("&", "\\&")
        return text.strip()

    @property
    def strip(self):
        return self.text.strip()

    def __str__(self):
        return self.text.strip()


class Body(TextChunk):
    def __init__(self, text):
        self.text = text

    @property
    def oneline(self):
        if "\n" not in self.text.strip():
            return self.text.strip()
        else:
            raise OrjiError(f"{self.text} is not one line")

    def tempfile(self):
        filepath = Path(f"/tmp/{random_5_digit_number()}.txt")
        filepath.write_text(self.text)
        return filepath.absolute()

    @property
    def paragraphs(self):
        return [
            TextChunk(text) for text in self.text.split("\n\n") if text.strip() != ""
        ]


class Note:
    def __init__(self, node):
        self._node = node

    @property
    def name(self):
        return self._node.heading

    @property
    def indexlookup(self):
        indices = []
        node = self._node
        while True:
            index = [i for i, n in enumerate(node.parent.children) if n == node][0]
            indices.append(str(index))
            if node.parent.is_root():
                break
            else:
                node = node.parent

        return "/".join(reversed(indices))

    @property
    def slug(self):
        return slugify(self._node.heading)

    @property
    def state(self):
        return self._node.todo

    @property
    def tags(self):
        return sorted(self._node.tags)

    @property
    def body(self):
        return Body(self._node.get_body(format="raw"))

    @property
    def prop(self):
        return self._node.properties

    def from_indexlookup(self, indexlookup):
        split = [int(x) for x in indexlookup.split("/")]
        node = self._node

        for index in split:
            node = node.children[index]

        return Note(node)

    def has(self, lookup):
        matching_notes = [n for n in self._node.children if n.heading == lookup]
        if len(matching_notes) == 0:
            return False
        elif len(matching_notes) > 1:
            raise OrjiError(
                f"More than one note found in {self.name} with name {lookup}"
            )
        else:
            return True

    def at(self, lookup):
        matching_notes = [n for n in self._node.children if n.heading == lookup]
        if len(matching_notes) == 0:
            raise OrjiError(f"No notes found in {self.name} with name {lookup}")
        elif len(matching_notes) > 1:
            raise OrjiError(
                f"More than one note found in {self.name} with name {lookup}"
            )
        else:
            return Note(matching_notes[0])

    def __iter__(self):
        for node in self._node.children:
            yield Note(node)



from .note import Note
from pathlib import Path
from orgparse import loads
import jinja2
import click
from click import echo
import imp
import inspect
from sys import exit
import stat
import subprocess


class Failure(Exception):
    pass


def fail(message):
    raise Failure(message)


def environment(latexmode, pymodule_filename):
    if latexmode:
        env = jinja2.Environment(
            block_start_string=r"\BLOCK{",
            block_end_string="}",
            variable_start_string=r"\VAR{",
            variable_end_string="}",
            comment_start_string=r"\#{",
            comment_end_string="}",
            line_statement_prefix="%%",
            line_comment_prefix="%#",
            trim_blocks=True,
            autoescape=False,
            undefined=jinja2.StrictUndefined,
            loader=jinja2.BaseLoader,
        )
    else:
        env = jinja2.Environment(
            undefined=jinja2.StrictUndefined, loader=jinja2.BaseLoader
        )

    if pymodule_filename is not None:
        pymodule_filepath = Path(pymodule_filename)

        if not pymodule_filepath.exists():
            click.echo(f"{pymodule_filename} not found", err=True)
            exit(1)

        module_contents = {
            key: item
            for key, item in inspect.getmembers(
                imp.load_source(
                    pymodule_filepath.stem, str(pymodule_filepath.absolute())
                )
            )
            if not key.startswith("_")
        }
        env.globals.update(module_contents)

    env.globals["fail"] = fail
    return env


@click.command()
@click.argument("orgdir")
@click.argument("rundir")
def run(orgdir, rundir):
    orgdir = Path(orgdir)
    rundir = Path(rundir)

    assert orgdir.is_dir()
    assert rundir.is_dir()
    assert len(list(orgdir.glob("*.org"))), "orgdir must contain org files"
    assert len(list(rundir.glob("*.sh"))), "rundir must contain sh files"

    scripts = {script.stem: script.read_text() for script in rundir.glob("*.sh")}
    env = environment(False, None)

    tmp = Path("/tmp")

    for orgfile in orgdir.glob("*.org"):
        for note in Note(loads(Path(orgfile).read_text())):
            if note.state == "TODO":
                for tag in note.tags:
                    if tag in scripts.keys():
                        notebody_path = tmp.joinpath("notebody.txt")
                        notebody_path.write_text(note.body.text)
                        tmp_script = tmp.joinpath("{}.sh".format(tag))

                        rendered_script = env.from_string(scripts[tag]).render(
                            notebody=notebody_path, note=note
                        )

                        tmp_script.write_text(rendered_script)
                        tmp_script.chmod(tmp_script.stat().st_mode | stat.S_IEXEC)
                        return_code = subprocess.call([tmp_script], shell=True)

                        if return_code != 0:
                            echo(scripts[tag])
                            echo("")
                            echo(rendered_script)
                            exit(1)



from .note import Note
from pathlib import Path
from orgparse import loads
import click
from .template import Template

# class Failure(Exception):
#     pass


# def fail(message):
#     raise Failure(message)


# def environment(latexmode, pymodule_filename):
#     if latexmode:
#         env = jinja2.Environment(
#             block_start_string=r"\BLOCK{",
#             block_end_string="}",
#             variable_start_string=r"\VAR{",
#             variable_end_string="}",
#             comment_start_string=r"\#{",
#             comment_end_string="}",
#             line_statement_prefix="%%",
#             line_comment_prefix="%#",
#             trim_blocks=True,
#             autoescape=False,
#             undefined=jinja2.StrictUndefined,
#             loader=jinja2.BaseLoader,
#         )
#     else:
#         env = jinja2.Environment(
#             undefined=jinja2.StrictUndefined, loader=jinja2.BaseLoader
#         )

#     if pymodule_filename is not None:
#         pymodule_filepath = Path(pymodule_filename)

#         if not pymodule_filepath.exists():
#             click.echo(f"{pymodule_filename} not found", err=True)
#             exit(1)

#         module_contents = {
#             key: item
#             for key, item in inspect.getmembers(
#                 imp.load_source(
#                     pymodule_filepath.stem, str(pymodule_filepath.absolute())
#                 )
#             )
#             if not key.startswith("_")
#         }
#         env.globals.update(module_contents)

#     env.globals["fail"] = fail
#     return env


# class Template:
#     def __init__(self, template_text, jinjafile, latexmode=None, pymodule_filename=None):
#         self._template_text = template_text
#         self._jinjafile = jinjafile
#         self._latexmode = latexmode
#         self._pymodule_filename = pymodule_filename
#         self._env = environment(latexmode, pymodule_filename)

#     def render(self, **vars):
#         try:
#             return self._env.from_string(self._template_text).render(**vars)
#         except jinja2.exceptions.UndefinedError as error:
#             lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
#             click.echo(f"Template error on line {lineno} of {self._jinjafile}: {error}", err=True)
#             exit(1)
#         except jinja2.exceptions.TemplateSyntaxError as error:
#             lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
#             click.echo(
#                 f"Template syntax error on line {lineno} of {self._jinjafile}: {error}", err=True
#             )
#             exit(1)
#         except jinja2.exceptions.TemplateRuntimeError as error:
#             lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
#             click.echo(
#                 f"Template runtime error on line {lineno} of {self._jinjafile}: {error}", err=True
#             )
#             exit(1)
#         except Failure as error:
#             lineno = traceback.extract_tb(error.__traceback__)[-2].lineno
#             click.echo(f"Failure on line {lineno} of {self._jinjafile}: {error}", err=True)
#             exit(1)
#         except OrjiError as error:
#             lineno = traceback.extract_tb(error.__traceback__)[-2].lineno
#             click.echo(f"Failure on line {lineno} of {self._jinjafile}: {error}", err=True)
#             exit(1)


@click.command()
@click.argument("orgfile")
@click.argument("jinjafile")
@click.option(
    "--indexlookup",
    "indexlookup",
    help="Specify zero-indexed subnote to use e.g. 0/0/4",
)
@click.option(
    "--latexmode",
    is_flag=True,
    show_default=True,
    default=False,
    help="Jinja2 latex mode",
)
@click.option(
    "--module",
    "pymodule",
    help="Specify python module to use in template.",
)
def cat(orgfile, jinjafile, indexlookup, latexmode, pymodule):
    org_text = Path(orgfile).read_text()
    template_text = Path(jinjafile).read_text()
    parsed = loads(org_text)
    notes = Note(parsed)

    if indexlookup is not None:
        notes = notes.from_indexlookup(indexlookup)

    output_text = Template(
        template_text, jinjafile, latexmode=latexmode, pymodule_filename=pymodule
    ).render(notes=notes, root=notes)

    # try:
    #     output_text = (
    #         environment(latexmode=latexmode, pymodule_filename=pymodule)
    #         .from_string(template_text)
    #         .render(notes=notes, root=notes)
    #     )
    # except jinja2.exceptions.UndefinedError as error:
    #     lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
    #     click.echo(f"Template error on line {lineno} of {jinjafile}: {error}", err=True)
    #     exit(1)
    # except jinja2.exceptions.TemplateSyntaxError as error:
    #     lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
    #     click.echo(
    #         f"Template syntax error on line {lineno} of {jinjafile}: {error}", err=True
    #     )
    #     exit(1)
    # except jinja2.exceptions.TemplateRuntimeError as error:
    #     lineno = traceback.extract_tb(error.__traceback__)[-1].lineno
    #     click.echo(
    #         f"Template runtime error on line {lineno} of {jinjafile}: {error}", err=True
    #     )
    #     exit(1)
    # except Failure as error:
    #     lineno = traceback.extract_tb(error.__traceback__)[-2].lineno
    #     click.echo(f"Failure on line {lineno} of {jinjafile}: {error}", err=True)
    #     exit(1)
    # except OrjiError as error:
    #     lineno = traceback.extract_tb(error.__traceback__)[-2].lineno
    #     click.echo(f"Failure on line {lineno} of {jinjafile}: {error}", err=True)
    #     exit(1)

    click.echo(output_text)



import random
import os

MOCK_RANDOM_FIX = 11110


def random_5_digit_number():
    if os.getenv("MOCK") == "yes":
        global MOCK_RANDOM_FIX
        MOCK_RANDOM_FIX = MOCK_RANDOM_FIX + 1
        return MOCK_RANDOM_FIX
    else:
        return random.randint(10000, 99999)
